/**
 * Copyright 2008-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.annotation.processor;

import java.lang.annotation.Annotation;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

/**
 * The {@link DefaultAnnotationProcessor} process an Object and call
 * {@link IAnnotationHandler}'s methods for each found annotation.
 * @author Guillaume Sauthier
 */
public class DefaultAnnotationProcessor implements IAnnotationProcessor {

	/**
	 * Registered {@link IAnnotationHandler}.
	 */
	private List<IAnnotationHandler> handlers;

	/**
	 * Construct a default {@link DefaultAnnotationProcessor}.
	 */
	public DefaultAnnotationProcessor() {
		handlers = new ArrayList<IAnnotationHandler>();
	}

	/**
	 * Process the given object.
	 * @param target processed object
	 * @throws ProcessorException
	 */
	public void process(final Object target) throws ProcessorException {
		// Extract the Object's Class and process it
		Class<?> clazz = target.getClass();
		process(target, clazz);
	}

	/**
	 * Process the given class and uses the given target
	 * instance for injection if any.
	 * @param target Object for injection
	 * @param clazz Class to be processed
	 * @throws ProcessorException
	 */
	public void process(final Object target, final Class<?> clazz) throws ProcessorException {

		// First process parent
		if (clazz.getSuperclass() != null) {
			process(target, clazz.getSuperclass());
		}

		// Process class level annotations
		processClassAnnotations(target, clazz);

		// Process method level annotations
		Method[] methods = clazz.getDeclaredMethods();
		for (Method method : methods) {
            processMethodAnnotations(target, method);
		}

		// Process field level annotations
		Field[] fields = clazz.getDeclaredFields();
		for (Field field : fields) {
			processFieldAnnotations(target, field);
		}
	}

	/**
	 * Process a field and its associated annotations
	 * @param target instance
	 * @param field processed field
	 * @throws ProcessorException
	 */
	protected void processFieldAnnotations(final Object target, final Field field) throws ProcessorException {

		// Extract annotations
		Annotation[] annotations = field.getAnnotations();
		if (annotations != null) {
			for (Annotation annotation : annotations) {
				for (IAnnotationHandler handler : handlers) {

					// If a suitable handler is found
					if (handler.isSupported(annotation.annotationType())) {
						handler.process(annotation, field, target);
					}
				}
			}
		}
	}

	/**
	 * Process a method and its associated annotations
	 * @param target instance
	 * @param method processed method
	 * @throws ProcessorException
	 */
	protected void processMethodAnnotations(final Object target, final Method method) throws ProcessorException {

		// Extract annotations
		Annotation[] annotations = method.getAnnotations();
		if (annotations != null) {
			for (Annotation annotation : annotations) {
				for (IAnnotationHandler handler : handlers) {

					// If a suitable handler is found
					if (handler.isSupported(annotation.annotationType())) {
						handler.process(annotation, method, target);
					}
				}
			}
		}
	}

	/**
	 * Process a class and its associated annotations
	 * @param target instance
	 * @param clazz processed class
	 * @throws ProcessorException
	 */
	protected void processClassAnnotations(final Object target, final Class<?> clazz) throws ProcessorException {

		// Extract annotations
		Annotation[] annotations = clazz.getAnnotations();
		if (annotations != null) {
			for (Annotation annotation : annotations) {
				for (IAnnotationHandler handler : handlers) {

					// If a suitable handler is found
					if (handler.isSupported(annotation.annotationType())) {
						handler.process(annotation, clazz, target);
					}
				}
			}
		}
	}

	/**
	 * Register the given handler
	 * @param handler registered handler
     */
	public void addAnnotationHandler(final IAnnotationHandler handler) {
        handlers.add(handler);
	}

	/**
	 * Unregister the given handler
	 * @param handler unregistered handler
	 */
	public void removeAnnotationHandler(final IAnnotationHandler handler) {
        handlers.remove(handler);
	}

}
