/**
 * Copyright 2008-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.annotation.processor;

import java.lang.annotation.Annotation;
import java.lang.reflect.Field;
import java.lang.reflect.Method;

/**
 * An {@link IAnnotationHandler} is responsible to do something
 * when a particular annotation is found.
 * @author Guillaume Sauthier
 */
public interface IAnnotationHandler {

	/**
	 * Declares that a given type of annotation is supported by this handler.
	 * @param clazz an annotation's type
	 * @return <code>true</code> if this handler supports the given annotation's type.
	 */
	boolean isSupported(Class<? extends Annotation> clazz);

	/**
	 * Process a class level annotation.
	 * This method is called by the processor when a supported annotation
	 * is found on a {@link Class}.
	 * @param annotation the found annotation
	 * @param clazz the class that supports the given annotation
	 * @param target target instance
	 * @throws ProcessorException
	 */
	void process(Annotation annotation, Class<?> clazz, Object target) throws ProcessorException;

	/**
	 * Process a method level annotation.
	 * This method is called by the processor when a supported annotation
	 * is found on a {@link Method}.
	 * @param annotation the found annotation
	 * @param method the method that supports the given annotation
	 * @param target target instance
	 * @throws ProcessorException
	 */
	void process(Annotation annotation, Method method, Object target) throws ProcessorException;

	/**
	 * Process a field level annotation.
	 * This method is called by the processor when a supported annotation
	 * is found on a {@link Field}.
	 * @param annotation the found annotation
	 * @param field the field that supports the given annotation
	 * @param target target instance
	 * @throws ProcessorException
	 */
	void process(Annotation annotation, Field field, Object target) throws ProcessorException;
}
