/**
 * Copyright 2008-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.annotation.processor.handler;

import org.ow2.util.annotation.processor.IAnnotationHandler;
import org.ow2.util.annotation.processor.ProcessorException;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

/**
 * An {@link AbstractInjectionHandler} is a simple {@link IAnnotationHandler}
 * that provides facility methods to perform field and method injection.
 *
 * @author Guillaume Sauthier
 */
public abstract class AbstractInjectionHandler extends AbstractAnnotationHandler {

    /**
     * Inject a given value in the given object, using the 'setter'
     * method provided as parameter.
     *
     * @param method setter method
     * @param target instance to be injected
     * @param value  value to be injected
     * @throws ProcessorException
     */
    protected void doInjectMethod(final Method method,
                                  final Object target,
                                  final Object value) throws ProcessorException {
        boolean accessibility = method.isAccessible();

        try {
            method.setAccessible(true);
            method.invoke(target, value);
        } catch (IllegalArgumentException e) {
            throw new ProcessorException("Cannot inject '" + method + "'", e);
        } catch (IllegalAccessException e) {
            throw new ProcessorException("Cannot inject '" + method + "'", e);
        } catch (InvocationTargetException e) {
            throw new ProcessorException("Cannot inject '" + method + "'", e);
        } finally {
            method.setAccessible(accessibility);
        }
    }

    /**
     * Inject a given value in the given object, using the field provided
     * as parameter.
     *
     * @param field  injection field
     * @param target instance to be injected
     * @param value  value to be injected
     * @throws ProcessorException
     */
    protected void doInjectField(final Field field,
                                 final Object target,
                                 final Object value) throws ProcessorException {
        boolean accessibility = field.isAccessible();

        try {
            field.setAccessible(true);
            field.set(target, value);
        } catch (IllegalArgumentException e) {
            throw new ProcessorException("Cannot inject '" + field + "'", e);
        } catch (IllegalAccessException e) {
            throw new ProcessorException("Cannot inject '" + field + "'", e);
        } finally {
            field.setAccessible(accessibility);
		}
	}

    /**
     * Compute the attribute name from a 'setter' method. Recognized patterns
     * are:
     * <ul>
     * <li>setAttribute()</li>
     * <li>unsetAttribute()</li>
     * <li>bindAttribute()</li>
     * <li>unbindAttribute()</li>
     * </ul>
     * <p/> The attribute name is then lower-cased (first char).
     * @param method base method
     * @return the attribute name (Java Bean property name)
     * @throws org.ow2.util.annotation.processor.ProcessorException if the method do not follow the recognized
     *         patterns
     */
    protected String getAttributeName(final Method method) throws ProcessorException {
        String name = method.getName();
        String attributeName = null;
        if (name.startsWith("set")) {
            attributeName = name.substring(3);
        } else if (name.startsWith("unset")) {
            attributeName = name.substring(5);
        } else if (name.startsWith("bind")) {
            attributeName = name.substring(4);
        } else if (name.startsWith("unbind")) {
            attributeName = name.substring(6);
        } else {
            throw new ProcessorException("Method '" + method + "' is not a recognised method");
        }

        attributeName = attributeName.substring(0, 1).toLowerCase() + attributeName.substring(1);

        return attributeName;
    }
    
    /**
     * @param method Method under test
     * @return <code>true</code> only if the given method is a setter method.
     */
    protected boolean isSetterMethod(final Method method) {
        
        // A setter method's name starts with "set"
        String name = method.getName();
        if (!name.startsWith("set")) {
            return false;
        }
        
        // A setter method only have 1 parameter
        Class<?>[] types = method.getParameterTypes();
        if (types.length != 1) {
            return false;
        }
        
        // Otherwise, that's a setter
        return true;
        
    }

    /**
     * @param method Method under test
     * @return the type of the setter method
     */
    protected Class<?> getSetterParameterType(final Method method) {

        if (!isSetterMethod(method)) {
            throw new IllegalStateException("Method " + method + " is not a setter method");
        }
        
        // A setter method only have 1 parameter
        return method.getParameterTypes()[0];
    }
}
